﻿namespace Performance
{
	public static class Test1
	{
		private sealed class Task
		{
			public static void SafeNew ()
			{
				Task obj = new Task();
				System.Threading.Thread t = new System.Threading.Thread(obj.SafeActivity);
				t.Start();
			}

			public static void UnsafeNew ()
			{
				Task obj = new Task();
				System.Threading.Thread t = new System.Threading.Thread(obj.UnsafeActivity);
				t.Start();
			}

			private unsafe void UnsafeActivity ()
			{
				const int n = 65535;
				int* a = stackalloc int[n];
				int* e = a + n;
				int* p, q;
				int i = 0;
				while (i < n)
				{
					a[i++] = i * 1524231 + 27736188;
				}
				p = a;
				while (p < e)
				{
					q = a;
					while (q < e)
					{
						if (*q < *p)
						{
							int t = *p;
							*p = *q;
							*q = t;
						}
						q++;
					}
					p++;
				}
				OnStop();
			}

			private void SafeActivity ()
			{
				const int n = 65535;
				int[] a = new int[n];
				int i, j;
				i = 0;
				while (i < n)
				{
					a[i++] = i * 1524231 + 27736188;
				}
				i = 0;
				while (i < n)
				{
					j = 0;
					while (j < n)
					{
						if (a[j] < a[i])
						{
							int t = a[i];
							a[i] = a[j];
							a[j] = t;
						}
						j++;
					}
					i++;
				}
				OnStop();
			}
		}

		private static System.DateTime startUtcTime;
		private static int activeThreadCount;
		private static int testThreadCount;
		private static double normalization;
		private static bool safe;
		private const string header = " threads |  mode  | normalized performance | absolute performance |";
		private static string line = "".PadLeft(header.Length, '-');

		private static void OnStop ()
		{
			if (System.Threading.Interlocked.Decrement(ref activeThreadCount) == 0)
			{
				double seconds = (System.DateTime.UtcNow - startUtcTime).TotalSeconds;
				//System.Console.WriteLine("seconds {0}", seconds);
				double p = testThreadCount / seconds;
				if (!(normalization > 0))
				{
					normalization = p;
				}
				string s0 = testThreadCount.ToString();
				string s1 = safe ? "  safe  " : " unsafe ";
				string s2 = (p / normalization).ToString("0.00");
				string s3 = (p / 0.0878).ToString("0.00");
				int i0 = header.IndexOf('|');
				int i1 = header.IndexOf('|', i0 + 1);
				int i2 = header.IndexOf('|', i1 + 1);
				int i3 = header.IndexOf('|', i2 + 1);
				System.Console.WriteLine("{0} |{1}|{2} |{3} |", s0.PadLeft(i0 - 1), s1, s2.PadLeft(i2 - i1 - 2), s3.PadLeft(i3 - i2 - 2));
				if (safe)
				{
					System.Console.WriteLine(line);
				}
				if (testThreadCount < 200)
				{
					safe = !safe;
					Next();
				}
			}
		}

		public static void Do ()
		{
			System.Console.WriteLine(line);
			System.Console.WriteLine(header);
			System.Console.WriteLine(line);
			Next();
		}

		private static void Next ()
		{
			if (!safe)
			{
				testThreadCount++;
			}
			activeThreadCount = testThreadCount;
			startUtcTime = System.DateTime.UtcNow;
			for (int i = 0; i < testThreadCount; i++)
			{
				if (safe)
				{
					Task.SafeNew();
				}
				else
				{
					Task.UnsafeNew();
				}
			}
		}
	}
}
